@@ -24,6 +24,7 @@
 #include <gmodule.h>
 #include <libgda/gda-config.h>
 #include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #include "gda-marshal.h"
 #include <string.h>
 #include <libgda/binreloc/gda-binreloc.h>
@@ -32,6 +33,7 @@
 #include <libgda/gda-set.h>
 #include <libgda/gda-holder.h>
 #include <libgda/gda-log.h>
+#include <libgda/gda-util.h>
 #ifdef HAVE_FAM
 #include <fam.h>
 #include <glib/giochannel.h>
@@ -53,7 +55,7 @@ struct _GdaConfigPrivate {
 	gchar *user_file;
 	gchar *system_file;
 	gboolean system_config_allowed;
-	GSList *dsn_list; /* list of GdaDataSourceInfo structures */
+	GSList *dsn_list; /* list of GdaDsnInfo structures */
 	GSList *prov_list; /* list of InternalProvider structures */
 };
 
@@ -73,11 +75,11 @@ static void gda_config_get_property (GObject *object,
 				     GParamSpec *pspec);
 static GdaConfig *unique_instance = NULL;
 
-static gint data_source_info_compare (GdaDataSourceInfo *infoa, GdaDataSourceInfo *infob);
-static void data_source_info_free (GdaDataSourceInfo *info);
+static gint data_source_info_compare (GdaDsnInfo *infoa, GdaDsnInfo *infob);
+static void data_source_info_free (GdaDsnInfo *info);
 static void internal_provider_free (InternalProvider *ip);
 static void load_config_file (const gchar *file, gboolean is_system);
-static void save_config_file (const gchar *file, gboolean is_system);
+static void save_config_file (gboolean is_system);
 static void load_all_providers (void);
 
 enum {
@@ -133,7 +135,7 @@ gda_config_class_init (GdaConfigClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	gda_config_signals[DSN_ADDED] =
-                g_signal_new ("dsn_added",
+                g_signal_new ("dsn-added",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaConfigClass, dsn_added),
@@ -141,7 +143,7 @@ gda_config_class_init (GdaConfigClass *klass)
                               gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
 	gda_config_signals[DSN_TO_BE_REMOVED] =
-                g_signal_new ("dsn_to_be_removed",
+                g_signal_new ("dsn-to-be-removed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaConfigClass, dsn_to_be_removed),
@@ -149,7 +151,7 @@ gda_config_class_init (GdaConfigClass *klass)
                               gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
 	gda_config_signals[DSN_REMOVED] =
-                g_signal_new ("dsn_removed",
+                g_signal_new ("dsn-removed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaConfigClass, dsn_removed),
@@ -157,7 +159,7 @@ gda_config_class_init (GdaConfigClass *klass)
                               gda_marshal_VOID__POINTER,
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
 	gda_config_signals[DSN_CHANGED] =
-                g_signal_new ("dsn_changed",
+                g_signal_new ("dsn-changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaConfigClass, dsn_changed),
@@ -169,12 +171,14 @@ gda_config_class_init (GdaConfigClass *klass)
         object_class->set_property = gda_config_set_property;
         object_class->get_property = gda_config_get_property;
 
+	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in ~/.libgda/config */
 	g_object_class_install_property (object_class, PROP_USER_FILE,
-                                         g_param_spec_string ("user_file", _("File to use for per-user DSN list"), 
+                                         g_param_spec_string ("user-filename", _("File to use for per-user DSN list"), 
 							      NULL, NULL,
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in $PREFIX/etc/libgda-4.0/config */
 	g_object_class_install_property (object_class, PROP_USER_FILE,
-                                         g_param_spec_string ("system_file", _("File to use for system-wide DSN list"), 
+                                         g_param_spec_string ("system-filename", _("File to use for system-wide DSN list"), 
 							      NULL, NULL,
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	
@@ -202,10 +206,9 @@ load_config_file (const gchar *file, gboolean is_system)
 	xmlNodePtr root;
 
 	doc = xmlParseFile (file);
-	if (!doc) {
-		g_warning (_("Could not load config file '%s'"), file);
+	if (!doc) 
 		return;
-	}
+
 	root = xmlDocGetRootElement (doc);
 	if (root) {
 		xmlNodePtr node;
@@ -215,7 +218,7 @@ load_config_file (const gchar *file, gboolean is_system)
 
 			xmlChar *prop;
 			gchar *ptr;
-			GdaDataSourceInfo *info;
+			GdaDsnInfo *info;
 			gboolean is_new = FALSE;
 			xmlNodePtr entry;
 			
@@ -228,9 +231,9 @@ load_config_file (const gchar *file, gboolean is_system)
 					break;
 				}
 			}
-			info = gda_config_get_dsn (ptr);
+			info = gda_config_get_dsn_info (ptr);
 			if (!info) {
-				info = g_new0 (GdaDataSourceInfo, 1);
+				info = g_new0 (GdaDsnInfo, 1);
 				info->name = g_strdup (ptr);
 				is_new = TRUE;
 			}
@@ -256,10 +259,16 @@ load_config_file (const gchar *file, gboolean is_system)
 					xmlFree (prop);
 					continue;
 				}
-				if (!strcmp ((gchar *) prop, "DSN"))
+				if (!strcmp ((gchar *) prop, "DSN")) 
 					info->cnc_string = g_strdup ((gchar *)value);
-				else if (!strcmp ((gchar *) prop, "Provider"))
-					info->provider = g_strdup ((gchar *)value);
+				else if (!strcmp ((gchar *) prop, "Provider")) {
+					GdaProviderInfo *pinfo;
+					pinfo = gda_config_get_provider_info ((gchar *)value);
+					if (pinfo)
+						info->provider = g_strdup (pinfo->id);
+					else
+						info->provider = g_strdup ((gchar *)value);
+				}
 				else if (!strcmp ((gchar *) prop, "Description"))
 					info->description = g_strdup ((gchar *)value);
 				if (!strcmp ((gchar *) prop, "Auth"))
@@ -272,18 +281,25 @@ load_config_file (const gchar *file, gboolean is_system)
 				xmlFree (value);
 			}
 			
-			if (username) {
+			if (username && *username) {
 				if (!info->auth_string) {
 					/* migrate username/password to auth_string */
-					if (password)
-						info->auth_string = g_strdup_printf ("USERNAME=%s;PASSWORD=%s",
-										     username, password);
+					gchar *s1;
+					s1 = gda_rfc1738_encode (username);
+					if (password) {
+						gchar *s2;
+						s2 = gda_rfc1738_encode (password);
+						info->auth_string = g_strdup_printf ("USERNAME=%s;PASSWORD=%s", s1, s2);
+						g_free (s2);
+					}
 					else
-						info->auth_string = g_strdup_printf ("USERNAME=%s", username);
+						info->auth_string = g_strdup_printf ("USERNAME=%s", s1);
+					g_free (s1);
 				}
-				g_free (username);
-				g_free (password);
 			}
+			g_free (username);
+			g_free (password);
+
 
 			/* signals */
 			if (is_new) {
@@ -299,7 +315,7 @@ load_config_file (const gchar *file, gboolean is_system)
 }
 
 static void
-save_config_file (const gchar *file, gboolean is_system)
+save_config_file (gboolean is_system)
 {
 	xmlDocPtr doc;
 	xmlNodePtr root;
@@ -317,7 +333,7 @@ save_config_file (const gchar *file, gboolean is_system)
 	root = xmlNewDocNode (doc, NULL, BAD_CAST "libgda-config", NULL);
         xmlDocSetRootElement (doc, root);
 	for (list = unique_instance->priv->dsn_list; list; list = list->next) {
-		GdaDataSourceInfo *info = (GdaDataSourceInfo*) list->data;
+		GdaDsnInfo *info = (GdaDsnInfo*) list->data;
 		if (info->is_system != is_system)
 			continue;
 
@@ -387,13 +403,13 @@ gda_config_constructor (GType type,
 								     construct_properties);
 		for (i = 0; i< n_construct_properties; i++) {
 			GObjectConstructParam *prop = &(construct_properties[i]);
-			if (!strcmp (g_param_spec_get_name (prop->pspec), "user_file")) {
+			if (!strcmp (g_param_spec_get_name (prop->pspec), "user-filename")) {
 				user_file_set = TRUE;
-				g_print ("GdaConfig user dir set\n");
+				/*g_print ("GdaConfig user dir set\n");*/
 			}
-			else if (!strcmp (g_param_spec_get_name (prop->pspec), "system_file")) {
+			else if (!strcmp (g_param_spec_get_name (prop->pspec), "system-filename")) {
 				system_file_set = TRUE;
-				g_print ("GdaConfig system dir set\n");
+				/*g_print ("GdaConfig system dir set\n");*/
 			}
 		}
 
@@ -402,9 +418,76 @@ gda_config_constructor (GType type,
 
 		/* define user and system dirs. if not already defined */
 		if (!user_file_set) {
-			if (g_get_home_dir ()) 
-				unique_instance->priv->user_file = g_build_filename (g_get_home_dir (),
-										    ".libgda", "config", NULL);
+			if (g_get_home_dir ()) {
+				gchar *confdir, *conffile;
+				gboolean setup_ok = TRUE;
+				confdir = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".libgda", NULL);
+				conffile = g_build_filename (g_get_home_dir (), ".libgda", "config", NULL);
+
+				if (!g_file_test (confdir, G_FILE_TEST_EXISTS)) {
+					if (g_mkdir (confdir, 0700))
+						{
+							setup_ok = FALSE;
+							g_warning (_("Error creating user specific "
+								     "configuration directory '%s'"), 
+								   confdir);
+						}
+					if (setup_ok) {
+						gchar *str;
+						gchar *full_file;
+						gsize len;
+#define DB_FILE "sales_test.db"
+#define DEFAULT_CONFIG \
+"<?xml version=\"1.0\"?>\n" \
+"<libgda-config>\n" \
+"    <section path=\"/apps/libgda/Datasources/SalesTest\">\n" \
+"        <entry name=\"DSN\" type=\"string\" value=\"DB_DIR=%s;DB_NAME=sales_test.db\"/>\n" \
+"        <entry name=\"Description\" type=\"string\" value=\"Test database for a sales department\"/>\n" \
+"        <entry name=\"Provider\" type=\"string\" value=\"SQLite\"/>\n" \
+"    </section>\n" \
+"</libgda-config>\n"
+#define DEFAULT_CONFIG_EMPTY \
+"<?xml version=\"1.0\"?>\n" \
+"<libgda-config>\n" \
+"    <!-- User specific data sources go here -->\n" \
+"</libgda-config>\n"
+
+						str = gda_gbr_get_file_path (GDA_ETC_DIR, 
+									     LIBGDA_ABI_NAME, DB_FILE, NULL);
+						if (g_file_get_contents (str, &full_file, &len, NULL)) {
+							gchar *dbfile;
+							
+							/* copy the Sales test database */
+							dbfile = g_build_filename (confdir, DB_FILE, NULL);
+							if (g_file_set_contents (dbfile, full_file, len, NULL)) {
+								gchar *str2;
+								str2 = g_strdup_printf (DEFAULT_CONFIG, confdir);
+								g_file_set_contents (conffile, str2, -1, NULL);
+								g_free (str2);
+							}
+							else
+								g_file_set_contents (conffile, DEFAULT_CONFIG_EMPTY, -1, NULL);
+							g_free (dbfile);
+							g_free (full_file);
+						}
+						else 
+							g_file_set_contents (conffile, DEFAULT_CONFIG_EMPTY, -1, NULL);
+						g_free (str);
+					}
+				}
+				else if (!g_file_test (confdir, G_FILE_TEST_IS_DIR)) {
+					setup_ok = FALSE;
+					g_warning (_("User specific "
+						     "configuration directory '%s' exists and is not a directory"), 
+						   confdir);
+				}
+				g_free (confdir);
+
+				if (setup_ok)
+					unique_instance->priv->user_file = conffile;
+				else
+					g_free (conffile);
+			}
 		}
 		if (!system_file_set) 
 			unique_instance->priv->system_file = gda_gbr_get_file_path (GDA_ETC_DIR, 
@@ -540,7 +623,10 @@ gda_config_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_config_init
 		};
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaConfig", &info, 0);
+		GDA_CONFIG_LOCK ();
+		if (type == 0)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaConfig", &info, 0);
+		GDA_CONFIG_UNLOCK ();
 	}
 
 	return type;
@@ -612,36 +698,51 @@ gda_config_get (void)
 }
 
 /**
- * gda_config_get_dsn
+ * gda_config_get_dsn_info
  * @dsn_name: the name of the DSN to look for
  *
- * Get information about the DSN named @dsn_name
+ * Get information about the DSN named @dsn_name. 
  *
- * Returns: a a pointer to read-only #GdaDataSourceInfo structure, or %NULL if not found
+ * @dsn_name's format is "[&lt;username&gt;[:&lt;password&gt;]@]&lt;DSN&gt;" (if &lt;username&gt;
+ * and optionaly &lt;password&gt; are provided, they are ignored). Also see the gda_dsn_split() utility
+ * function.
+ *
+ * Returns: a a pointer to read-only #GdaDsnInfo structure, or %NULL if not found
  */
-GdaDataSourceInfo *
-gda_config_get_dsn (const gchar *dsn_name)
+GdaDsnInfo *
+gda_config_get_dsn_info (const gchar *dsn_name)
 {
 	GSList *list;
 
 	g_return_val_if_fail (dsn_name, NULL);
 
+	gchar *user, *pass, *real_dsn;
+        gda_dsn_split (dsn_name, &real_dsn, &user, &pass);
+	g_free (user);
+	g_free (pass);
+        if (!real_dsn) {
+		g_warning (_("Malformed data source name '%s'"), dsn_name);
+                return NULL;
+	}
+
 	GDA_CONFIG_LOCK ();
 	if (!unique_instance)
 		gda_config_get ();
 
 	for (list = unique_instance->priv->dsn_list; list; list = list->next)
-		if (!strcmp (((GdaDataSourceInfo*) list->data)->name, dsn_name)) {
+		if (!strcmp (((GdaDsnInfo*) list->data)->name, real_dsn)) {
 			GDA_CONFIG_UNLOCK ();
-			return (GdaDataSourceInfo*) list->data;
+			g_free (real_dsn);
+			return (GdaDsnInfo*) list->data;
 		}
 	GDA_CONFIG_UNLOCK ();
+	g_free (real_dsn);
 	return NULL;
 }
 
 /**
  * gda_config_define_dsn
- * @info: a pointer to a filled GdaDataSourceInfo structure
+ * @info: a pointer to a filled GdaDsnInfo structure
  * @error: a place to store errors, or %NULL
  *
  * Add or update a DSN from the definition in @info
@@ -649,9 +750,9 @@ gda_config_get_dsn (const gchar *dsn_name)
  * Returns: TRUE if no error occurred
  */
 gboolean
-gda_config_define_dsn (const GdaDataSourceInfo *info, GError **error)
+gda_config_define_dsn (const GdaDsnInfo *info, GError **error)
 {
-	GdaDataSourceInfo *einfo;
+	GdaDsnInfo *einfo;
 	gboolean save_user = FALSE;
 	gboolean save_system = FALSE;
 
@@ -673,7 +774,7 @@ gda_config_define_dsn (const GdaDataSourceInfo *info, GError **error)
 		save_system = TRUE;
 	else
 		save_user = TRUE;
-	einfo = gda_config_get_dsn (info->name);
+	einfo = gda_config_get_dsn_info (info->name);
 	if (einfo) {
 		g_free (einfo->provider); einfo->provider = NULL;
 		g_free (einfo->cnc_string); einfo->cnc_string = NULL;
@@ -696,7 +797,7 @@ gda_config_define_dsn (const GdaDataSourceInfo *info, GError **error)
 		g_signal_emit (unique_instance, gda_config_signals[DSN_CHANGED], 0, einfo);
 	}
 	else {
-		einfo = g_new0 (GdaDataSourceInfo, 1);
+		einfo = g_new0 (GdaDsnInfo, 1);
 		einfo->name = g_strdup (info->name);
 		if (info->provider)
 			einfo->provider = g_strdup (info->provider);
@@ -714,9 +815,9 @@ gda_config_define_dsn (const GdaDataSourceInfo *info, GError **error)
 	}
 	
 	if (save_system)
-		save_config_file (unique_instance->priv->system_file, TRUE);
+		save_config_file (TRUE);
 	if (save_user)
-		save_config_file (unique_instance->priv->user_file, FALSE);
+		save_config_file (FALSE);
 
 	GDA_CONFIG_UNLOCK ();
 	return TRUE;
@@ -734,7 +835,7 @@ gda_config_define_dsn (const GdaDataSourceInfo *info, GError **error)
 gboolean
 gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 {
-	GdaDataSourceInfo *info;
+	GdaDsnInfo *info;
 	gboolean save_user = FALSE;
 	gboolean save_system = FALSE;
 
@@ -744,7 +845,7 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 	if (!unique_instance)
 		gda_config_get ();
 
-	info = gda_config_get_dsn (dsn_name);
+	info = gda_config_get_dsn_info (dsn_name);
 	if (!info) {
 		g_set_error (error, GDA_CONFIG_ERROR, GDA_CONFIG_DSN_NOT_FOUND_ERROR,
 			     _("Unknown DSN '%s'"), dsn_name);
@@ -769,14 +870,43 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 	data_source_info_free (info);
 	
 	if (save_system)
-		save_config_file (unique_instance->priv->system_file, TRUE);
+		save_config_file (TRUE);
 	if (save_user)
-		save_config_file (unique_instance->priv->user_file, FALSE);
+		save_config_file (FALSE);
 
 	GDA_CONFIG_UNLOCK ();
 	return TRUE;
 }
 
+/**
+ * gda_config_dsn_needs_auth
+ * @dsn_name: the name of a DSN, in the "[&lt;username&gt;[:&lt;password&gt;]@]&lt;DSN&gt;" format
+ * 
+ * Tells if the data source identified as @dsn_name needs any authentication. If a &lt;username&gt;
+ * and optionaly a &lt;password&gt; are specified, they are ignored.
+ *
+ * Returns: TRUE if an authentication is needed
+ */
+gboolean
+gda_config_dsn_needs_authentication (const gchar *dsn_name)
+{
+	GdaDsnInfo *info;
+	GdaProviderInfo *pinfo;
+
+	info = gda_config_get_dsn_info (dsn_name);
+	if (!info)
+		return FALSE;
+	pinfo = gda_config_get_provider_info (info->provider);
+	if (!pinfo) {
+		g_warning (_("Provider '%s' not found"), info->provider);
+		return FALSE;
+	}
+	if (pinfo->auth_params && pinfo->auth_params->holders)
+		return TRUE;
+	else
+		return FALSE;
+}
+
 /**
  * gda_config_list_dsn
  * 
@@ -828,7 +958,7 @@ gda_config_get_nb_dsn (void)
 }
 
 /**
- * gda_config_get_dsn_index
+ * gda_config_get_dsn_info_index
  * @dsn_name:
  * 
  * Get the index (starting at 0) of the DSN named @dsn_name
@@ -836,9 +966,9 @@ gda_config_get_nb_dsn (void)
  * Returns: the index or -1 if not found
  */
 gint
-gda_config_get_dsn_index (const gchar *dsn_name)
+gda_config_get_dsn_info_index (const gchar *dsn_name)
 {
-	GdaDataSourceInfo *info;
+	GdaDsnInfo *info;
 	gint ret = -1;
 
 	g_return_val_if_fail (dsn_name, -1);
@@ -846,7 +976,7 @@ gda_config_get_dsn_index (const gchar *dsn_name)
 	if (!unique_instance)
 		gda_config_get ();
 
-	info = gda_config_get_dsn (dsn_name);
+	info = gda_config_get_dsn_info (dsn_name);
 	if (info)
 		ret = g_slist_index (unique_instance->priv->dsn_list, info);
 
@@ -855,17 +985,17 @@ gda_config_get_dsn_index (const gchar *dsn_name)
 }
 
 /**
- * gda_config_get_dsn_at_index
+ * gda_config_get_dsn_info_at_index
  * @index:
  *
- * Get a pointer to a read-only #GdaDataSourceInfo at the @index position
+ * Get a pointer to a read-only #GdaDsnInfo at the @index position
  *
  * Returns: the pointer or %NULL if no DSN exists at position @index
  */
-GdaDataSourceInfo *
-gda_config_get_dsn_at_index (gint index)
+GdaDsnInfo *
+gda_config_get_dsn_info_at_index (gint index)
 {
-	GdaDataSourceInfo *ret;
+	GdaDsnInfo *ret;
 	GDA_CONFIG_LOCK ();
 	if (!unique_instance)
 		gda_config_get ();
@@ -915,6 +1045,11 @@ gda_config_get_provider_info (const gchar *provider_name)
 	if (!unique_instance->priv->prov_list) 
 		load_all_providers ();
 
+	if (!g_ascii_strcasecmp (provider_name, "MS Access")) {
+		GDA_CONFIG_UNLOCK ();
+		return gda_config_get_provider_info ("MSAccess");
+	}
+
 	for (list = unique_instance->priv->prov_list; list; list = list->next)
 		if (!g_ascii_strcasecmp (((GdaProviderInfo*) list->data)->id, provider_name)) {
 			GDA_CONFIG_UNLOCK ();
@@ -925,17 +1060,18 @@ gda_config_get_provider_info (const gchar *provider_name)
 }
 
 /**
- * gda_config_get_provider_object
+ * gda_config_get_provider
  * @provider_name:
  * @error: a place to store errors, or %NULL
  *
  * Get a pointer to the session-wide #GdaServerProvider for the
- * provider named @provider_name
+ * provider named @provider_name. The caller must not call g_object_unref() on the
+ * returned object.
  *
  * Returns: a pointer to the #GdaServerProvider, or %NULL if an error occurred
  */
 GdaServerProvider *
-gda_config_get_provider_object (const gchar *provider_name, GError **error)
+gda_config_get_provider (const gchar *provider_name, GError **error)
 {
 	InternalProvider *ip;
 	GdaServerProvider  *(*plugin_create_provider) (void);
@@ -959,14 +1095,14 @@ gda_config_get_provider_object (const gchar *provider_name, GError **error)
 	g_module_symbol (ip->handle, "plugin_create_provider", (gpointer) &plugin_create_provider);
 	if (!plugin_create_provider) {
 		g_set_error (error, GDA_CONFIG_ERROR, GDA_CONFIG_PROVIDER_CREATION_ERROR,
-			     _("Can't instanciate provider '%s'"), provider_name);
+			     _("Can't instantiate provider '%s'"), provider_name);
 		GDA_CONFIG_UNLOCK ();
 		return NULL;
 	}
 	ip->instance = plugin_create_provider ();
 	if (!ip->instance) {
 		g_set_error (error, GDA_CONFIG_ERROR, GDA_CONFIG_PROVIDER_CREATION_ERROR,
-			     _("Can't instanciate provider '%s'"), provider_name);
+			     _("Can't instantiate provider '%s'"), provider_name);
 		GDA_CONFIG_UNLOCK ();
 		return NULL;
 	}
@@ -985,8 +1121,8 @@ gda_config_get_provider_object (const gchar *provider_name, GError **error)
  *  <listitem><para>Provider name</para></listitem>
  *  <listitem><para>Description</para></listitem>
  *  <listitem><para>DSN parameters</para></listitem>
- *  <listitem><para>Authentification parameters</para></listitem>
- *  <listitem><para>File</para></listitem>
+ *  <listitem><para>Authentication parameters</para></listitem>
+ *  <listitem><para>File name of the plugin</para></listitem>
  * </itemizedlist>
  *
  * Returns: a new #GdaDataModel
@@ -1013,7 +1149,7 @@ gda_config_list_providers (void)
 	gda_data_model_set_column_title (model, 0, _("Provider"));
 	gda_data_model_set_column_title (model, 1, _("Description"));
 	gda_data_model_set_column_title (model, 2, _("DSN parameters"));
-	gda_data_model_set_column_title (model, 3, _("Authentification"));
+	gda_data_model_set_column_title (model, 3, _("Authentication"));
 	gda_data_model_set_column_title (model, 4, _("File"));
 	g_object_set_data (G_OBJECT (model), "name", _("List of installed providers"));
 
@@ -1085,16 +1221,18 @@ load_all_providers (void)
 	const gchar *dirname;
 	g_assert (unique_instance);
 
-	dirname = getenv ("GDA_PROVIDERS_ROOT_DIR");
-	if (dirname)
-		load_providers_from_dir (dirname, TRUE);
+	dirname = g_getenv ("GDA_TOP_BUILD_DIR");
+	if (dirname) {
+		gchar *pdir;
+		pdir = g_build_path (G_DIR_SEPARATOR_S, dirname, "providers", NULL);
+		load_providers_from_dir (pdir, TRUE);
+	}
 	else {
 		gchar *str;
 		str = gda_gbr_get_file_path (GDA_LIB_DIR, LIBGDA_ABI_NAME, "providers", NULL);
 		load_providers_from_dir (str, FALSE);
 		g_free (str);
 	}
-
 }
 
 static void 
@@ -1120,7 +1258,6 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 		GdaProviderInfo *info;
 		GModule *handle;
 		gchar *path;
-		gchar *ext;
 		void (*plugin_init) (const gchar *);
 		const gchar * (* plugin_get_name) (void);
 		const gchar * (* plugin_get_description) (void);
@@ -1135,10 +1272,13 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 			g_free (cname);
 		}
 
-		ext = g_strrstr (name, ".");
-		if (!ext)
+		if (!g_str_has_suffix (name, "." G_MODULE_SUFFIX))
 			continue;
-		if (strcmp (ext + 1, G_MODULE_SUFFIX))
+#ifdef G_WITH_CYGWIN
+		if (!g_str_has_prefix (name, "cyggda"))
+#else
+		if (!g_str_has_prefix (name, "libgda"))
+#endif
 			continue;
 
 		path = g_build_path (G_DIR_SEPARATOR_S, dirname,
@@ -1195,12 +1335,13 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 				   forget about that provider... */
 				internal_provider_free (ip);
 				ip = NULL;
+				continue;
 			}
 		}
 		else
 			g_warning ("Provider '%s' does not provide a DSN spec", info->id);
 
-		/* Authentification parameters */
+		/* Authentication parameters */
 		info->auth_params = NULL;
 		if (plugin_get_auth_spec) {
 			GError *error = NULL;
@@ -1223,6 +1364,7 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 				   forget about that provider... */
 				internal_provider_free (ip);
 				ip = NULL;
+				continue;
 			}
 		}
 		else {
@@ -1250,22 +1392,30 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 
 /* sorting function */
 static gint
-data_source_info_compare (GdaDataSourceInfo *infoa, GdaDataSourceInfo *infob)
+data_source_info_compare (GdaDsnInfo *infoa, GdaDsnInfo *infob)
 {
 	if (!infoa && !infob)
 		return 0;
 	if (infoa) {
 		if (!infob)
 			return 1;
-		else
-			return strcmp (infoa->name, infob->name);
+		else {
+			gchar *u8a, *u8b;
+			gint res;
+			u8a = g_utf8_casefold (infoa->name, -1);
+			u8b = g_utf8_casefold (infob->name, -1);
+			res = strcmp (u8a, u8b);
+			g_free (u8a);
+			g_free (u8b);
+			return res;
+		}
 	}
 	else
 		return -1;
 }
 
 static void 
-data_source_info_free (GdaDataSourceInfo *info)
+data_source_info_free (GdaDsnInfo *info)
 {
 	g_free (info->provider); 
 	g_free (info->cnc_string); 
@@ -1286,7 +1436,8 @@ internal_provider_free (InternalProvider *ip)
 	g_free (info->id);
 	g_free (info->location);
 	g_free (info->description);
-	g_object_unref (info->dsn_params);
+	if (info->dsn_params)
+		g_object_unref (info->dsn_params);
 	g_free (ip);
 }
 
@@ -1342,12 +1493,12 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
                         g_print ("Reloading config files (%s config has changed)\n", is_system ? "global" : "user");
 			GSList *list;
 			for (list = unique_instance->priv->dsn_list; list; list = list->next) {
-				GdaDataSourceInfo *info = (GdaDataSourceInfo *) list->data;
+				GdaDsnInfo *info = (GdaDsnInfo *) list->data;
 				g_print ("[info %p]: %s/%s\n", info, info->provider, info->name);
 			}
 #endif
 			while (unique_instance->priv->dsn_list) {
-				GdaDataSourceInfo *info = (GdaDataSourceInfo *) unique_instance->priv->dsn_list->data;
+				GdaDsnInfo *info = (GdaDsnInfo *) unique_instance->priv->dsn_list->data;
 				g_signal_emit (unique_instance, gda_config_signals[DSN_TO_BE_REMOVED], 0, info);
 				unique_instance->priv->dsn_list = g_slist_remove (unique_instance->priv->dsn_list, info);
 				g_signal_emit (unique_instance, gda_config_signals[DSN_REMOVED], 0, info);
