@@ -56,10 +56,10 @@
 /* VOID:OBJECT,OBJECT (gda-marshal.list:28) */
 void
 gda_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
-                                 GValue       *return_value,
+                                 GValue       *return_value G_GNUC_UNUSED,
                                  guint         n_param_values,
                                  const GValue *param_values,
-                                 gpointer      invocation_hint,
+                                 gpointer      invocation_hint G_GNUC_UNUSED,
                                  gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer     data1,
@@ -93,10 +93,10 @@ gda_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
 /* VOID:OBJECT,ENUM,OBJECT (gda-marshal.list:29) */
 void
 gda_marshal_VOID__OBJECT_ENUM_OBJECT (GClosure     *closure,
-                                      GValue       *return_value,
+                                      GValue       *return_value G_GNUC_UNUSED,
                                       guint         n_param_values,
                                       const GValue *param_values,
-                                      gpointer      invocation_hint,
+                                      gpointer      invocation_hint G_GNUC_UNUSED,
                                       gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_ENUM_OBJECT) (gpointer     data1,
@@ -132,10 +132,10 @@ gda_marshal_VOID__OBJECT_ENUM_OBJECT (GClosure     *closure,
 /* VOID:OBJECT,UINT (gda-marshal.list:30) */
 void
 gda_marshal_VOID__OBJECT_UINT (GClosure     *closure,
-                               GValue       *return_value,
+                               GValue       *return_value G_GNUC_UNUSED,
                                guint         n_param_values,
                                const GValue *param_values,
-                               gpointer      invocation_hint,
+                               gpointer      invocation_hint G_GNUC_UNUSED,
                                gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_UINT) (gpointer     data1,
@@ -169,10 +169,10 @@ gda_marshal_VOID__OBJECT_UINT (GClosure     *closure,
 /* VOID:OBJECT,UINT,UINT (gda-marshal.list:31) */
 void
 gda_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
-                                    GValue       *return_value,
+                                    GValue       *return_value G_GNUC_UNUSED,
                                     guint         n_param_values,
                                     const GValue *param_values,
-                                    gpointer      invocation_hint,
+                                    gpointer      invocation_hint G_GNUC_UNUSED,
                                     gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_UINT_UINT) (gpointer     data1,
@@ -208,10 +208,10 @@ gda_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
 /* VOID:OBJECT,BOOLEAN (gda-marshal.list:32) */
 void
 gda_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
-                                  GValue       *return_value,
+                                  GValue       *return_value G_GNUC_UNUSED,
                                   guint         n_param_values,
                                   const GValue *param_values,
-                                  gpointer      invocation_hint,
+                                  gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_BOOLEAN) (gpointer     data1,
@@ -245,10 +245,10 @@ gda_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
 /* VOID:STRING,UINT,UINT (gda-marshal.list:33) */
 void
 gda_marshal_VOID__STRING_UINT_UINT (GClosure     *closure,
-                                    GValue       *return_value,
+                                    GValue       *return_value G_GNUC_UNUSED,
                                     guint         n_param_values,
                                     const GValue *param_values,
-                                    gpointer      invocation_hint,
+                                    gpointer      invocation_hint G_GNUC_UNUSED,
                                     gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_UINT_UINT) (gpointer     data1,
@@ -284,10 +284,10 @@ gda_marshal_VOID__STRING_UINT_UINT (GClosure     *closure,
 /* VOID:ENUM,OBJECT (gda-marshal.list:34) */
 void
 gda_marshal_VOID__ENUM_OBJECT (GClosure     *closure,
-                               GValue       *return_value,
+                               GValue       *return_value G_GNUC_UNUSED,
                                guint         n_param_values,
                                const GValue *param_values,
-                               gpointer      invocation_hint,
+                               gpointer      invocation_hint G_GNUC_UNUSED,
                                gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__ENUM_OBJECT) (gpointer     data1,
@@ -323,10 +323,10 @@ gda_marshal_VOID__ENUM_OBJECT (GClosure     *closure,
 /* VOID:INT,INT (gda-marshal.list:36) */
 void
 gda_marshal_VOID__INT_INT (GClosure     *closure,
-                           GValue       *return_value,
+                           GValue       *return_value G_GNUC_UNUSED,
                            guint         n_param_values,
                            const GValue *param_values,
-                           gpointer      invocation_hint,
+                           gpointer      invocation_hint G_GNUC_UNUSED,
                            gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_INT) (gpointer     data1,
@@ -360,10 +360,10 @@ gda_marshal_VOID__INT_INT (GClosure     *closure,
 /* VOID:INT,BOXED,BOXED (gda-marshal.list:37) */
 void
 gda_marshal_VOID__INT_BOXED_BOXED (GClosure     *closure,
-                                   GValue       *return_value,
+                                   GValue       *return_value G_GNUC_UNUSED,
                                    guint         n_param_values,
                                    const GValue *param_values,
-                                   gpointer      invocation_hint,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_BOXED_BOXED) (gpointer     data1,
@@ -399,10 +399,10 @@ gda_marshal_VOID__INT_BOXED_BOXED (GClosure     *closure,
 /* VOID:INT,BOOLEAN (gda-marshal.list:38) */
 void
 gda_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
-                               GValue       *return_value,
+                               GValue       *return_value G_GNUC_UNUSED,
                                guint         n_param_values,
                                const GValue *param_values,
-                               gpointer      invocation_hint,
+                               gpointer      invocation_hint G_GNUC_UNUSED,
                                gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__INT_BOOLEAN) (gpointer     data1,
@@ -436,10 +436,10 @@ gda_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
 /* VOID:STRING,INT (gda-marshal.list:39) */
 void
 gda_marshal_VOID__STRING_INT (GClosure     *closure,
-                              GValue       *return_value,
+                              GValue       *return_value G_GNUC_UNUSED,
                               guint         n_param_values,
                               const GValue *param_values,
-                              gpointer      invocation_hint,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__STRING_INT) (gpointer     data1,
@@ -473,10 +473,10 @@ gda_marshal_VOID__STRING_INT (GClosure     *closure,
 /* VOID:ENUM,POINTER (gda-marshal.list:40) */
 void
 gda_marshal_VOID__ENUM_POINTER (GClosure     *closure,
-                                GValue       *return_value,
+                                GValue       *return_value G_GNUC_UNUSED,
                                 guint         n_param_values,
                                 const GValue *param_values,
-                                gpointer      invocation_hint,
+                                gpointer      invocation_hint G_GNUC_UNUSED,
                                 gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__ENUM_POINTER) (gpointer     data1,
@@ -507,62 +507,23 @@ gda_marshal_VOID__ENUM_POINTER (GClosure     *closure,
             data2);
 }
 
-/* BOOLEAN:INT (gda-marshal.list:41) */
+/* POINTER:INT,INT (gda-marshal.list:41) */
 void
-gda_marshal_BOOLEAN__INT (GClosure     *closure,
-                          GValue       *return_value,
-                          guint         n_param_values,
-                          const GValue *param_values,
-                          gpointer      invocation_hint,
-                          gpointer      marshal_data)
-{
-  typedef gboolean (*GMarshalFunc_BOOLEAN__INT) (gpointer     data1,
-                                                 gint         arg_1,
-                                                 gpointer     data2);
-  register GMarshalFunc_BOOLEAN__INT callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gboolean v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 2);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_BOOLEAN__INT) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       g_marshal_value_peek_int (param_values + 1),
-                       data2);
-
-  g_value_set_boolean (return_value, v_return);
-}
-
-/* BOOLEAN:INT,INT (gda-marshal.list:42) */
-void
-gda_marshal_BOOLEAN__INT_INT (GClosure     *closure,
-                              GValue       *return_value,
+gda_marshal_POINTER__INT_INT (GClosure     *closure,
+                              GValue       *return_value G_GNUC_UNUSED,
                               guint         n_param_values,
                               const GValue *param_values,
-                              gpointer      invocation_hint,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT) (gpointer     data1,
+  typedef gpointer (*GMarshalFunc_POINTER__INT_INT) (gpointer     data1,
                                                      gint         arg_1,
                                                      gint         arg_2,
                                                      gpointer     data2);
-  register GMarshalFunc_BOOLEAN__INT_INT callback;
+  register GMarshalFunc_POINTER__INT_INT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
-  gboolean v_return;
+  gpointer v_return;
 
   g_return_if_fail (return_value != NULL);
   g_return_if_fail (n_param_values == 3);
@@ -577,23 +538,60 @@ gda_marshal_BOOLEAN__INT_INT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_BOOLEAN__INT_INT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_POINTER__INT_INT) (marshal_data ? marshal_data : cc->callback);
 
   v_return = callback (data1,
                        g_marshal_value_peek_int (param_values + 1),
                        g_marshal_value_peek_int (param_values + 2),
                        data2);
 
-  g_value_set_boolean (return_value, v_return);
+  g_value_set_pointer (return_value, v_return);
+}
+
+/* POINTER:VOID (gda-marshal.list:42) */
+void
+gda_marshal_POINTER__VOID (GClosure     *closure,
+                           GValue       *return_value G_GNUC_UNUSED,
+                           guint         n_param_values,
+                           const GValue *param_values,
+                           gpointer      invocation_hint G_GNUC_UNUSED,
+                           gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_POINTER__VOID) (gpointer     data1,
+                                                  gpointer     data2);
+  register GMarshalFunc_POINTER__VOID callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 1);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_POINTER__VOID) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       data2);
+
+  g_value_set_pointer (return_value, v_return);
 }
 
 /* POINTER:POINTER (gda-marshal.list:43) */
 void
 gda_marshal_POINTER__POINTER (GClosure     *closure,
-                              GValue       *return_value,
+                              GValue       *return_value G_GNUC_UNUSED,
                               guint         n_param_values,
                               const GValue *param_values,
-                              gpointer      invocation_hint,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
   typedef gpointer (*GMarshalFunc_POINTER__POINTER) (gpointer     data1,
@@ -626,3 +624,83 @@ gda_marshal_POINTER__POINTER (GClosure     *closure,
   g_value_set_pointer (return_value, v_return);
 }
 
+/* BOOLEAN:POINTER (gda-marshal.list:44) */
+void
+gda_marshal_BOOLEAN__POINTER (GClosure     *closure,
+                              GValue       *return_value G_GNUC_UNUSED,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
+                              gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER) (gpointer     data1,
+                                                     gpointer     arg_1,
+                                                     gpointer     data2);
+  register GMarshalFunc_BOOLEAN__POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_pointer (param_values + 1),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* POINTER:OBJECT,POINTER (gda-marshal.list:45) */
+void
+gda_marshal_POINTER__OBJECT_POINTER (GClosure     *closure,
+                                     GValue       *return_value G_GNUC_UNUSED,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint G_GNUC_UNUSED,
+                                     gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_POINTER__OBJECT_POINTER) (gpointer     data1,
+                                                            gpointer     arg_1,
+                                                            gpointer     arg_2,
+                                                            gpointer     data2);
+  register GMarshalFunc_POINTER__OBJECT_POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_POINTER__OBJECT_POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_object (param_values + 1),
+                       g_marshal_value_peek_pointer (param_values + 2),
+                       data2);
+
+  g_value_set_pointer (return_value, v_return);
+}
+
