@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2006 The GNOME Foundation.
+ * Copyright (C) 2006 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -23,14 +23,14 @@
 #include <stdlib.h>
 #include <glib/gmessages.h>
 #include <libgda/gda-marshal.h>
-#include <libgda/gda-dict.h>
-#include <libgda/gda-dict-type.h>
 #include <libgda/gda-server-provider.h>
 #include <libgda/gda-server-operation.h>
-#include <libgda/gda-parameter-list.h>
-#include <libgda/gda-parameter.h>
+#include <libgda/gda-set.h>
+#include <libgda/gda-holder.h>
 #include <libgda/gda-connection.h>
-#include <libgda/gda-data-model-private.h> /* For #include <libgda/gda-dict.h>() */
+#include <libgda/gda-data-model-private.h>
+#include <libgda/gda-data-model-import.h>
+#include <libgda/gda-data-model-array.h>
 #include "gda-util.h"
 #include <string.h>
 #ifdef HAVE_LOCALE_H
@@ -38,7 +38,8 @@
 #endif
 #include <glib/gi18n-lib.h>
 
-#define PARENT_TYPE G_TYPE_OBJECT
+extern gchar *gda_lang_locale;
+
 #define CLASS(operation) (GDA_SERVER_OPERATION_CLASS (G_OBJECT_GET_CLASS (operation)))
 
 static void gda_server_operation_class_init (GdaServerOperationClass *klass);
@@ -69,7 +70,6 @@ static gint gda_server_operation_signals[LAST_SIGNAL] = { 0, 0 };
 enum
 {
 	PROP_0,
-	PROP_DICT,
 	PROP_CNC,
 	PROP_PROV,
 	PROP_OP_TYPE,
@@ -85,9 +85,9 @@ typedef struct _Node {
 	GdaServerOperationNodeStatus  status;
 	gchar                        *path_name; /* NULL for GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM nodes */
 	union {
-		GdaParameterList     *plist;
+		GdaSet               *plist;
 		GdaDataModel         *model;
-		GdaParameter         *param; 
+		GdaHolder            *param; 
 		struct {
 			GSList       *seq_tmpl; /* list of Node templates */
 			guint         min_items;
@@ -112,7 +112,6 @@ static gchar *node_get_complete_path (GdaServerOperation *op, Node *node);
 static void   clean_nodes_info_cache (GdaServerOperation *operation); 
 struct _GdaServerOperationPrivate {
 	GdaServerOperationType  op_type;
-	GdaDict                *dict;
 	gboolean                cnc_set;
 	GdaConnection          *cnc;
 	gboolean                prov_set;
@@ -140,7 +139,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 
 	/* signals */
 	gda_server_operation_signals[SEQUENCE_ITEM_ADDED] =
-		g_signal_new ("sequence_item_added",
+		g_signal_new ("sequence-item-added",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_added),
@@ -148,7 +147,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
 	gda_server_operation_signals[SEQUENCE_ITEM_REMOVE] =
-		g_signal_new ("sequence_item_remove",
+		g_signal_new ("sequence-item-remove",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_remove),
@@ -165,24 +164,20 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 	object_class->set_property = gda_server_operation_set_property;
 	object_class->get_property = gda_server_operation_get_property;
 
-	g_object_class_install_property (object_class, PROP_DICT,
-					 g_param_spec_object ("dict", NULL, NULL, 
-                                                               GDA_TYPE_DICT,
-							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_CNC,
 					 g_param_spec_object ("connection", NULL, NULL, 
-                                                               GDA_TYPE_CONNECTION,
-							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+							      GDA_TYPE_CONNECTION,
+							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_PROV,
-					 g_param_spec_object ("provider_obj", NULL, NULL, 
-                                                               GDA_TYPE_SERVER_PROVIDER,
-							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_object ("provider", NULL, NULL, 
+							      GDA_TYPE_SERVER_PROVIDER,
+							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_SPEC_FILE,
-					 g_param_spec_string ("spec_file", NULL, NULL, 
+					 g_param_spec_string ("spec-filename", NULL, NULL, 
 							      NULL, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OP_TYPE,
-					 g_param_spec_int ("op_type", NULL, NULL, 
-							   0, GDA_SERVER_OPERATION_NB - 1, 
+					 g_param_spec_int ("op-type", NULL, NULL, 
+							   0, GDA_SERVER_OPERATION_LAST - 1, 
 							   0, G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
@@ -219,8 +214,6 @@ gda_server_operation_dispose (GObject *object)
 		if (operation->priv->info_hash)
 			g_hash_table_destroy (operation->priv->info_hash);
 
-		if (operation->priv->dict)
-			g_object_unref (operation->priv->dict);
 		if (operation->priv->cnc)
 			g_object_unref (operation->priv->cnc);
 		if (operation->priv->prov)
@@ -230,8 +223,7 @@ gda_server_operation_dispose (GObject *object)
 			node_destroy (operation, NODE (operation->priv->topnodes->data));
 		g_assert (!operation->priv->allnodes);
 
-		if (operation->priv->xml_spec_doc)
-			xmlFreeDoc (operation->priv->xml_spec_doc);
+		/* don't free operation->priv->xml_spec_doc */
 
 		if (operation->priv->sources) {
 			g_slist_foreach (operation->priv->sources, (GFunc) g_object_unref, NULL);
@@ -251,7 +243,9 @@ gda_server_operation_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+
 		static const GTypeInfo info = {
 			sizeof (GdaServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -264,7 +258,10 @@ gda_server_operation_get_type (void)
 			(GInstanceInitFunc) gda_server_operation_init
 		};
 
-		type = g_type_register_static (PARENT_TYPE, "GdaServerOperation", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (!type)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaServerOperation", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -335,8 +332,10 @@ node_destroy (GdaServerOperation *op, Node *node)
 	}
 
 	g_free (node->path_name);
-	op->priv->topnodes = g_slist_remove (op->priv->topnodes, node);
-	op->priv->allnodes = g_slist_remove (op->priv->allnodes, node);
+	if (op) {
+		op->priv->topnodes = g_slist_remove (op->priv->topnodes, node);
+		op->priv->allnodes = g_slist_remove (op->priv->allnodes, node);
+	}
 
 	g_free (node);
 }
@@ -421,6 +420,7 @@ node_find_or_create (GdaServerOperation *op, const gchar *path)
 				node = NULL;
 				break;
 		}
+		g_free(cpath);
 	}
 
 	/*g_print ("# %s (%s) => %p\n", __FUNCTION__, path, node);*/
@@ -466,7 +466,7 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 	return retval;
 }
 
-static GSList *load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root);
+static GSList *load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, GError **error);
 
 /* add a new item to @node and inserts it into @op's private structures */
 static void
@@ -474,6 +474,7 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 {
 	gchar *path, *seq_path;
 	Node *new_node;
+	GSList *seq_item_nodes;
 
 	g_assert (node);
 	g_assert (node->type == GDA_SERVER_OPERATION_NODE_SEQUENCE);
@@ -481,13 +482,16 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 	seq_path = node_get_complete_path (op, node);
 	path = g_strdup_printf ("%s/%d", seq_path, g_slist_length (node->d.seq.seq_items));
 
+	seq_item_nodes = load_xml_spec (op, node->d.seq.xml_spec, path, NULL);
+	g_assert (seq_item_nodes);
+
 	new_node = node_new (node, GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM, NULL);
 	op->priv->allnodes = g_slist_append (op->priv->allnodes, new_node);
 	new_node->d.seq_item_nodes = NULL;
 	new_node->status = node->status;
 	node->d.seq.seq_items = g_slist_append (node->d.seq.seq_items, new_node);
 
-	new_node->d.seq_item_nodes = load_xml_spec (op, node->d.seq.xml_spec, path);
+	new_node->d.seq_item_nodes = seq_item_nodes;
 
 	clean_nodes_info_cache (op);
 #ifdef GDA_DEBUG_signal
@@ -516,27 +520,20 @@ gda_server_operation_set_property (GObject *object,
 	op = GDA_SERVER_OPERATION (object);
 	if (op->priv) {
 		switch (param_id) {
-		case PROP_DICT:
-			if (op->priv->dict)
-				g_object_unref (op->priv->dict);
-
-			op->priv->dict = ASSERT_DICT (g_value_get_object (value));
-			g_object_ref (op->priv->dict);
-			break;
 		case PROP_CNC:
 			if (op->priv->cnc)
 				g_object_unref (op->priv->cnc);
 
-			op->priv->cnc = GDA_CONNECTION( g_value_get_object (value));
+			op->priv->cnc = GDA_CONNECTION (g_value_get_object (value));
 			op->priv->cnc_set = TRUE;
 
 			if (op->priv->cnc) {
 				g_object_ref (op->priv->cnc);
 
-				if (gda_connection_get_provider_obj (op->priv->cnc)) {
+				if (gda_connection_get_provider (op->priv->cnc)) {
 					if (op->priv->prov)
 						g_object_unref (op->priv->prov);
-					op->priv->prov = gda_connection_get_provider_obj (op->priv->cnc);
+					op->priv->prov = gda_connection_get_provider (op->priv->cnc);
 					g_object_ref (op->priv->prov);
 					op->priv->prov_set = TRUE;
 				}
@@ -557,12 +554,24 @@ gda_server_operation_set_property (GObject *object,
 		case PROP_SPEC_FILE: {
 			xmlDocPtr doc;
 			const gchar *xmlfile;
+			static GHashTable *doc_hash = NULL; /* key = file name, value = xmlDocPtr */
 
 			xmlDtdPtr old_dtd = NULL;
 			xmlfile = g_value_get_string (value);
 			if (!xmlfile)
 				return;
 
+			if (!doc_hash)
+				doc_hash = g_hash_table_new_full (g_str_hash, g_str_equal, 
+								  g_free, (GDestroyNotify) xmlFreeDoc);
+			else {
+				doc = g_hash_table_lookup (doc_hash, xmlfile);
+				if (doc) {
+					op->priv->xml_spec_doc = doc;
+					break;
+				}
+			}
+
 			if (! g_file_test (xmlfile, G_FILE_TEST_EXISTS)) {
 				g_warning (_("GdaServerOperation: could not find file '%s'"), xmlfile);
 				return;
@@ -587,7 +596,7 @@ gda_server_operation_set_property (GObject *object,
 					old_dtd = doc->intSubset;
 					doc->intSubset = gda_server_op_dtd;
 				}
-
+#ifndef G_OS_WIN32
 				if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
 					gchar *str;
 					
@@ -609,12 +618,14 @@ gda_server_operation_set_property (GObject *object,
 					xmlDoValidityCheckingDefaultValue = xmlcheck;
 					return;
 				}
+#endif
 				
 				xmlDoValidityCheckingDefaultValue = xmlcheck;
 				g_free (validc);
 				if (gda_server_op_dtd)
 					doc->intSubset = old_dtd;
 				op->priv->xml_spec_doc = doc;
+				g_hash_table_insert (doc_hash, g_strdup (xmlfile), doc);
 			}
 			else {
 				g_warning (_("GdaServerOperation: could not load file '%s'"), xmlfile);
@@ -627,9 +638,17 @@ gda_server_operation_set_property (GObject *object,
 		}
 	}
 
-	if (op->priv->xml_spec_doc && op->priv->dict && op->priv->cnc_set && op->priv->prov_set) 
+	if (op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) {
 		/* load XML file */
-		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL);
+		GError *lerror = NULL;
+		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL, &lerror);
+		if (!op->priv->topnodes) {
+			g_warning (_("Could not load XML specifications: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+		}
+	}
 }
 
 static void
@@ -643,6 +662,12 @@ gda_server_operation_get_property (GObject *object,
 	op = GDA_SERVER_OPERATION (object);
 	if (op->priv) {
 		switch (param_id) {
+		case PROP_CNC:
+			g_value_set_object (value, op->priv->cnc);
+			break;
+		case PROP_PROV:
+			g_value_set_object (value, op->priv->prov);
+			break;
 		case PROP_OP_TYPE:
 			g_value_set_int (value, op->priv->op_type);
 			break;
@@ -678,21 +703,15 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
 }
 
 static GSList *
-load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
+load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, GError **error)
 {
 	xmlNodePtr node;
-	const gchar *lang;
+	const gchar *lang = gda_lang_locale;
 	GSList *retlist = NULL;
 	Node *parent = NULL;
 
 	if (root) 
 		parent = node_find (op, root);
-
-#ifdef HAVE_LC_MESSAGES
-        lang = setlocale (LC_MESSAGES, NULL);
-#else
-        lang = setlocale (LC_CTYPE, NULL);
-#endif
 	
 	g_assert (specnode);
 
@@ -703,7 +722,7 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 		node = specnode->children;
 		while (node && (xmlNodeIsText (node) || strcmp ((gchar*)node->name, "sources"))) 
 		node = node->next; 
-		if (node && !strcmp ((gchar*)node->name, "sources")){
+		if (node && !strcmp ((gchar*)node->name, "sources")) {
 			for (node = node->xmlChildrenNode; (node != NULL); node = node->next) {
 				if (xmlNodeIsText (node)) 
 					continue;
@@ -716,16 +735,18 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 					errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (model));
 					if (errors) {
 						g_object_unref (model);
-						model = NULL;
+						if (sources) {
+							g_slist_foreach (sources, (GFunc) g_object_unref, NULL);
+							g_slist_free (sources);
+							return NULL;
+						}
 					}
 					else  {
 						xmlChar *str;
 						sources = g_slist_prepend (sources, model);
-						str = xmlGetProp(node, (xmlChar*)"name");
-						if (str) {
-							gda_object_set_name (GDA_OBJECT (model), (gchar*)str);
-							xmlFree (str);
-						}
+						str = xmlGetProp (node, (xmlChar*)"name");
+						if (str) 
+							g_object_set_data_full (G_OBJECT (model), "name", (gchar*)str, xmlFree);
 					}
 				}
 			}
@@ -738,12 +759,15 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 	while (node) {
 		xmlChar *id, *this_lang;
 		gchar *complete_path = NULL, *path_name = NULL;
-		GError *error = NULL;
 		Node *opnode = NULL;
 		Node *old_opnode;
 
 		if (xmlNodeIsText (node)) {
-			node = node->next;
+			xmlNodePtr nextnode;
+			nextnode = node->next;
+			xmlUnlinkNode (node);
+			xmlFreeNode (node);
+			node = nextnode;
 			continue;
 		}
 
@@ -751,12 +775,16 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 		this_lang = xmlGetProp(node, (xmlChar*)"lang");
 		if (this_lang) {
 			if (strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
+				xmlNodePtr nextnode;
 				xmlFree (this_lang);
-				node = node->next;
+				nextnode = node->next;
+				xmlUnlinkNode (node);
+				xmlFreeNode (node);
+				node = nextnode;
 				continue;
 			}
 
-			xmlFree(this_lang);
+			xmlFree (this_lang);
 		}
 
 		id = xmlGetProp (node, BAD_CAST "id");
@@ -777,15 +805,10 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 
 		/* GDA_SERVER_OPERATION_NODE_PARAMLIST */
 		if (!strcmp ((gchar*)node->name, "parameters")) {
-			GdaParameterList *plist;
+			GdaSet *plist;
 
-			plist = gda_parameter_list_new_from_spec_node (NULL, node, &error);
-			if (!plist) {
-				g_warning (_("GdaServerOperation: error loading GdaParameterList spec %s: '%s'"), 
-					   complete_path, error && error->message ? error->message : _("No detail"));
-				g_error_free (error);
-			}
-			else {
+			plist = gda_set_new_from_spec_node (node, error);
+			if (plist) {
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAMLIST, path_name);
 				opnode->d.plist = plist;
 			}
@@ -795,12 +818,9 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 			GdaDataModel *import;
 
 			import = gda_data_model_import_new_xml_node (node);
-			if (!import) {
-				TO_IMPLEMENT;
-			}
-			else {
+			if (import) {
 				GdaDataModel *model;
-				model = gda_data_model_array_copy_model (import, NULL);
+				model = (GdaDataModel*) gda_data_model_array_copy_model (import, NULL);
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_DATA_MODEL, path_name);
 				opnode->d.model = model;
 				g_object_unref (import);
@@ -844,45 +864,39 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 				xmlFree (prop);
 			}
 
-			seq_tmpl = load_xml_spec (op, node, complete_path);
-			opnode->d.seq.seq_tmpl = seq_tmpl;
+			seq_tmpl = load_xml_spec (op, node, complete_path, error);
+			if (! seq_tmpl) {
+				node_destroy (NULL, opnode);
+				opnode = NULL;
+			}
+			else
+				opnode->d.seq.seq_tmpl = seq_tmpl;
 		}
 
 		/* GDA_SERVER_OPERATION_NODE_PARAM */
 		else if (!strcmp ((gchar*)node->name, "parameter")) {
-			GdaParameter *param = NULL;
-			GdaDictType *dtype = NULL;
-			xmlChar *dbmstype, *gdatype;
-			xmlChar *str;
-			gboolean dtype_created = FALSE;
+			GdaHolder *param = NULL;
+			xmlChar *gdatype;
 
-			/* find data type and create GdaParameter */
-			dbmstype = xmlGetProp (node, BAD_CAST "dbmstype");
+			/* find data type and create GdaHolder */
 			gdatype = xmlGetProp (node, BAD_CAST "gdatype");
-			dtype = gda_utility_find_or_create_data_type (op->priv->dict, op->priv->prov, op->priv->cnc,
-								  (gchar*)dbmstype, (gchar*)gdatype, &dtype_created);
-			if (dbmstype) xmlFree (dbmstype);
-			if (gdatype) xmlFree (gdatype);
-			if (!dtype) {
-				str = xmlGetProp (node, BAD_CAST "name");
-				g_warning (_("Can't find a data type for parameter '%s'"), 
-					   str ? (gchar *) str : _("unnamed"));
-				xmlFree (str);
-			}
-			else {
-				param = GDA_PARAMETER (g_object_new (GDA_TYPE_PARAMETER, "dict", op->priv->dict,
-								     "g_type", gda_dict_type_get_g_type (dtype),
-								     NULL));
-				if (dtype_created)
-					g_object_unref (dtype);
+			param = GDA_HOLDER (g_object_new (GDA_TYPE_HOLDER,
+							  "g-type", 
+							  gdatype ? gda_g_type_from_string ((gchar*) gdatype) : G_TYPE_STRING,
+							  NULL));
+			if (gdatype)
+				xmlFree (gdatype);
 			
-				/* set parameter's attributes */
-				gda_utility_parameter_load_attributes (param, node, op->priv->sources);
-
+			/* set parameter's attributes */
+			if (gda_utility_holder_load_attributes (param, node, op->priv->sources, error)) {
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAM, path_name);
 				opnode->d.param = param;
 			}
 		}
+		else {
+			node = node->next;
+			continue;
+		}
 		
 		/* really insert the new Node, and set its status */
 		if (opnode) {
@@ -911,7 +925,12 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 				}
 			}
 		}
-		
+		else {
+			g_free (path_name);
+			g_free (complete_path);
+			g_slist_free (retlist);
+			return NULL;
+		}
 		g_free (path_name);
 		g_free (complete_path);
 		node = node->next;
@@ -924,6 +943,9 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
  * gda_server_operation_new
  * @xml_file: a file which has the specifications for the GdaServerOperation object to create
  *
+ * IMPORTANT NOTE: Using this funtion is not the recommended way of creating a #GdaServerOperation object, the
+ * correct way is to use gda_server_provider_create_operation(); this method is reserved for internal implementation.
+ *
  * Creates a new #GdaServerOperation object from the @xml_file specifications
  *
  * The @xml_file must respect the DTD described in the "libgda-server-operation.dtd" file: its top
@@ -936,7 +958,19 @@ gda_server_operation_new (GdaServerOperationType op_type, const gchar *xml_file)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op_type", op_type, "spec_file", xml_file, NULL);
+	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op-type", op_type, "spec-filename", xml_file, NULL);
+#ifdef GDA_DEBUG_NO
+	{
+		g_print ("New GdaServerOperation:\n");
+		xmlNodePtr node;
+		node = gda_server_operation_save_data_to_xml ((GdaServerOperation *) obj, NULL);
+		xmlDocPtr doc;
+		doc = xmlNewDoc ("1.0");
+		xmlDocSetRootElement (doc, node);
+		xmlDocDump (stdout, doc);
+		xmlFreeDoc (doc);
+	}
+#endif
 	return (GdaServerOperation *) obj;
 }
 
@@ -1008,9 +1042,9 @@ gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_fo
 			g_free (str);
 		}
 		if (node && (node->type == GDA_SERVER_OPERATION_NODE_PARAMLIST)) {
-			GdaParameter *param;
+			GdaHolder *param;
 			extension = gda_server_operation_get_node_path_portion (op, path);
-			param = gda_parameter_list_find_param (node->d.plist, extension);
+			param = gda_set_get_holder (node->d.plist, extension);
 			g_free (extension);
 
 			if (param) {
@@ -1036,6 +1070,7 @@ gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_fo
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, extension +1))
 						column = NULL;
+					g_free(colid);
 				}
 			}
 			g_free (extension);
@@ -1103,13 +1138,18 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 		return "ADD_COLUMN";
         case GDA_SERVER_OPERATION_DROP_COLUMN:
 		return "DROP_COLUMN";
+	case GDA_SERVER_OPERATION_CREATE_VIEW:
+		return "CREATE_VIEW";
+	case GDA_SERVER_OPERATION_DROP_VIEW:
+		return "DROP_VIEW";
 	default:
-		g_error (_("Non handled GdaServerOperationType, please report error"));
+		g_error (_("Non handled GdaServerOperationType, please report error to "
+			   "http://bugzilla.gnome.org/ for the \"libgda\" product"));
 		return "";
 	}
 }
 
-static void node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent);
+static gboolean node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent);
 
 /**
  * gda_server_operation_save_data_to_xml
@@ -1132,19 +1172,24 @@ gda_server_operation_save_data_to_xml (GdaServerOperation *op, GError **error)
 	g_return_val_if_fail (op->priv, NULL);
 
 	topnode = xmlNewNode (NULL, BAD_CAST "serv_op_data");
+	xmlSetProp (topnode, BAD_CAST "type", 
+		    BAD_CAST gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (op)));
 
-	list = op->priv->topnodes;
-	while (list) {
-		node_save (op, NODE (list->data), topnode);
-		list = g_slist_next (list);
+	for (list = op->priv->topnodes; list; list = list->next) {
+		if (!node_save (op, NODE (list->data), topnode)) {
+			xmlFreeNode (topnode);
+			topnode = NULL;
+			break;
+		}
 	}
 
 	return topnode;
 }
 
-static void
+static gboolean
 node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 {
+	gboolean retval = TRUE;
 	xmlNodePtr node;
 	GSList *list;
 	gchar *complete_path;
@@ -1153,13 +1198,13 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	complete_path = node_get_complete_path (op, opnode);
 	switch (opnode->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST:
-		list = opnode->d.plist->parameters;
+		list = opnode->d.plist->holders;
 		while (list) {
 			gchar *path;
 			const GValue *value;
 			gchar *str;
 
-			value = gda_parameter_get_value (GDA_PARAMETER (list->data));
+			value = gda_holder_get_value (GDA_HOLDER (list->data));
 			if (!value || gda_value_is_null ((GValue *) value))
 				str = NULL;
 			else {
@@ -1171,7 +1216,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 			node = xmlNewChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
 			g_free (str);
 
-			path = g_strdup_printf ("%s/%s", complete_path, gda_object_get_id (GDA_OBJECT (list->data)));
+			path = g_strdup_printf ("%s/%s", complete_path, gda_holder_get_id (GDA_HOLDER (list->data)));
 			xmlSetProp(node, (xmlChar*)"path", (xmlChar*)path);
 			g_free (path);
 
@@ -1181,13 +1226,14 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
 		node = xmlNewChild (parent, NULL, BAD_CAST "op_data", NULL);
 		xmlSetProp(node, (xmlChar*)"path", (xmlChar*)complete_path);
-		gda_utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE);
+		if (!gda_utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE))
+			retval = FALSE;
 		break;
 	case GDA_SERVER_OPERATION_NODE_PARAM: {
 		const GValue *value;
 		gchar *str;
 		
-		value = gda_parameter_get_value (opnode->d.param);
+		value = gda_holder_get_value (opnode->d.param);
 		if (!value || gda_value_is_null ((GValue *) value))
 			str = NULL;
 		else {
@@ -1205,14 +1251,20 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 		GSList *list;
 		
 		for (list =  opnode->d.seq.seq_items; list; list = list->next) 
-			node_save (op, NODE (list->data), parent);
+			if (!node_save (op, NODE (list->data), parent)) {
+				retval = FALSE;
+				break;
+			}
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM: {
 		GSList *list;
 		
 		for (list =  opnode->d.seq_item_nodes; list; list = list->next) 
-			node_save (op, NODE (list->data), parent);
+			if (! node_save (op, NODE (list->data), parent)) {
+				retval = FALSE;
+				break;
+			}
 		break;
 	}
 	default:
@@ -1220,6 +1272,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	}
 
 	g_free (complete_path);
+	return retval;
 }
 
 /**
@@ -1274,7 +1327,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 	/* actual data loading */
 	if (strcmp ((gchar*)node->name, "serv_op_data")) {
 		g_set_error (error, 0, 0,
-			     _("Expected tag <serv_op_data>, got <%s>"), node->name);
+			     _("Expected tag <%s>, got <%s>"), "serv_op_data", node->name);
 		return FALSE;
 	}
 	
@@ -1288,7 +1341,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 
 		if (strcmp ((gchar*)cur->name, "op_data")) {
 			g_set_error (error, 0, 0,
-				     _("Expected tag <op_data>, got <%s>"), cur->name);
+				     _("Expected tag <%s>, got <%s>"), "op_data", cur->name);
 			return FALSE;
 		}
 
@@ -1327,32 +1380,35 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 						
 						contents = cur->children;
 						if (contents && xmlNodeIsText (contents)) {
-							GdaParameter *param;
-							param = gda_parameter_list_find_param (opnode->d.plist, extension);
-							if (param && 
-							    !gda_parameter_set_value_str (param, (gchar*)contents->content)) {
-								g_set_error (error, 0, 0,
-									     _("Could not set parameter '%s' to value '%s'"), 
-									     prop, cur->content);
-								allok = FALSE;
+							GdaHolder *param;
+							param = gda_set_get_holder (opnode->d.plist, extension);
+							if (param) {
+								GValue *v;
+								v = gda_value_new_from_string ((gchar*)contents->content, 
+											       gda_holder_get_g_type (param));
+								if (!gda_holder_take_value (param, v, error)) 
+									allok = FALSE;
 							}
 						}
 					}
 					break;
 				case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
 					gda_data_model_array_clear (GDA_DATA_MODEL_ARRAY (opnode->d.model));
-					if (! gda_data_model_add_data_from_xml_node (opnode->d.model, cur->children, error))
+					if (cur->children && 
+						 ! gda_data_model_add_data_from_xml_node (opnode->d.model, 
+											  cur->children, error))
 						allok = FALSE;
 					break;
 				case GDA_SERVER_OPERATION_NODE_PARAM: {
 					xmlNodePtr contents;
 
 					contents = cur->children;
-					if (contents && xmlNodeIsText (contents) && 
-					    !gda_parameter_set_value_str (opnode->d.param, (gchar*)contents->content)) {
-						g_set_error (error, 0, 0,
-							     _("Could not set parameter '%s' to value '%s'"), prop, cur->content);
-						allok = FALSE;
+					if (contents && xmlNodeIsText (contents)) {
+						GValue *v;
+						v = gda_value_new_from_string ((gchar*)contents->content, 
+									       gda_holder_get_g_type (opnode->d.param));
+						if (!gda_holder_take_value (opnode->d.param, v, error)) 
+							allok = FALSE;
 					}
 					break;
 				}
@@ -1799,7 +1855,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 	if (node_info) {
 		switch (node_info->type) {
 		case GDA_SERVER_OPERATION_NODE_PARAM:
-			value = gda_parameter_get_value (node_info->param);
+			value = gda_holder_get_value (node_info->param);
 			break;
 		case GDA_SERVER_OPERATION_NODE_PARAMLIST:
 		case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
@@ -1828,7 +1884,8 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 				if (row >= 0) 
 					value = gda_data_model_get_value_at (node_info->model, 
 									     gda_column_get_position (node_info->column), 
-									     row);
+									     row, NULL);
+				g_free(extension);
 			}
 			g_free (str);
 		}		
@@ -1846,12 +1903,16 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
  * @path_format: a complete path to a node (starting with "/")
  * @...: arguments to use with @path_format to make a complete path
  *
- * Set the value for the node at the path formed using @path_format and ... the rules are the same as
- * for g_strdup_printf())
+ * Set the value for the node at the path formed using @path_format and @... the rules are the same as
+ * for g_strdup_printf()). 
  *
- * Here are the corner cases:
+ * Note that trying to set a value for a path which is not used by the current
+ * provider (such as "/TABLE_OPTIONS_P/TABLE_ENGINE" for a PostgreSQL connection), will <emphasis>not</emphasis> generate
+ * any error; this allows one to set all the possible parameters and use the same code for several providers.
+ *
+ * Here are the possible formats of @path_format:
  * <itemizedlist>
- *  <listitem><para>If the path corresponds to a #GdaParameter, then the parameter is set to @value</para></listitem>
+ *  <listitem><para>If the path corresponds to a #GdaHolder, then the parameter is set to @value</para></listitem>
  *  <listitem><para>If the path corresponds to a sequence item like for example "/SEQUENCE_NAME/5/NAME" for
  *     the "NAME" value of the 6th item of the "SEQUENCE_NAME" sequence then:
  *     <itemizedlist>
@@ -1861,7 +1922,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
  *           the value to the corresponding in the 6th item of the sequence</para></listitem>
  *     </itemizedlist>
  *  </para></listitem>
- *  <listitem><para>If the path corresponds to a #GdaDataModel, like for example "/ARRAY/@COLUMN/5" for the value at the
+ *  <listitem><para>If the path corresponds to a #GdaDataModel, like for example "/ARRAY/@@COLUMN/5" for the value at the
  *     6th row of the "COLUMN" column of the "ARRAY" data model, then:
  *     <itemizedlist>
  *        <listitem><para>if the data model already contains 6 or more rows, then the value is just set</para></listitem>
@@ -1934,14 +1995,17 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 				allok = FALSE;
 			}
 			else {
-				GdaParameter *param;
-				param = gda_parameter_list_find_param (opnode->d.plist, extension);
-				if (param && 
-				    !gda_parameter_set_value_str (param, value)) {
-					g_set_error (error, 0, 0,
-						     _("Could not set parameter '%s' to value '%s'"), 
-						     path, value);
-					allok = FALSE;
+				GdaHolder *param;
+				param = gda_set_get_holder (opnode->d.plist, extension);
+				if (param) {
+					GValue *v;
+					if (value)
+						v = gda_value_new_from_string (value, 
+									       gda_holder_get_g_type (param));
+					else
+						v = gda_value_new_null ();
+					if (!gda_holder_take_value (param, v, error))
+						allok = FALSE;
 				}
 			}
 			break;
@@ -1958,6 +2022,7 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, colname +1))
 						column = NULL;
+					g_free(colid);
 				}
 				if (column) {
 					gchar *ptr;
@@ -1976,8 +2041,11 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 						
 						if (allok) {
 							GValue *gvalue;
-							gvalue = gda_value_new_from_string (value, 
-											    gda_column_get_g_type (column));
+							if (value)
+								gvalue = gda_value_new_from_string (value, 
+												    gda_column_get_g_type (column));
+							else
+								gvalue = gda_value_new_null ();
 							allok = gda_data_model_set_value_at (opnode->d.model,
 											     gda_column_get_position (column), 
 											     row, gvalue, error);
@@ -1989,12 +2057,14 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 			break;
 		}
 		case GDA_SERVER_OPERATION_NODE_PARAM: {
-			if (!gda_parameter_set_value_str (opnode->d.param, value)) {
-				g_set_error (error, 0, 0,
-					     _("Could not set parameter '%s' to value '%s'"), 
-					     path, value);
+			GValue *v;
+			if (value)
+				v = gda_value_new_from_string (value, 
+							       gda_holder_get_g_type (opnode->d.param));
+			else
+				v = gda_value_new_null ();
+			if (!gda_holder_take_value (opnode->d.param, v, error))
 				allok = FALSE;
-			}
 			break;
 		}
 		case GDA_SERVER_OPERATION_NODE_SEQUENCE:
@@ -2008,13 +2078,14 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 
 	g_free (extension);
 	g_free (colname);
+	g_free (path);
 	return allok;
 }
 
 /**
  * gda_server_operation_is_valid
  * @op: a #GdaServerOperation widget
- * @xml_file: a XML specification file (see gda_server_operation_new())
+ * @xml_file: an XML specification file (see gda_server_operation_new())
  * @error: a place to store an error, or %NULL
  *
  * Tells if all the required values in @op have been defined.
@@ -2035,8 +2106,7 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 
 	if (!xml_file) {
 		/* basic validity test */
-		list = op->priv->allnodes;
-		while (list && valid) {
+		for (list = op->priv->allnodes; list; list = list->next) {
 			Node *node;
 			
 			node = NODE (list->data);
@@ -2051,22 +2121,16 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 						valid = FALSE;
 						g_set_error (error, 0, 0,
 							     _("Missing required value for '%s'"), path);
+						break;
 					}
 					g_free (path);
 				}
 				else if (node->type == GDA_SERVER_OPERATION_NODE_PARAMLIST) {
-					valid = gda_parameter_list_is_valid (node->d.plist);
-					if (!valid) {
-						gchar *path;
-
-						path = node_get_complete_path (op, node);
-						g_set_error (error, 0, 0,
-							     _("Missing required value for list of parameters '%s'"), path);
-						g_free (path);
-					}
+					valid = gda_set_is_valid (node->d.plist, error);
+					if (!valid)
+						break;
 				}
 			}
-			list = list->next;
 		}
 	}
 	else {
