@@ -43,6 +43,8 @@
 
 #include <libgda/binreloc/gda-binreloc.h>
 
+extern gchar *gda_lang_locale;
+
 /**
  * gda_g_type_to_string
  * @type: Type to convert from.
@@ -78,6 +80,10 @@ gda_g_type_from_string (const gchar *str)
 			type = G_TYPE_STRING;
 		else if (!g_ascii_strcasecmp (str, "date"))
 			type = G_TYPE_DATE;
+		else if (!g_ascii_strcasecmp (str, "time"))
+			type = GDA_TYPE_TIME;
+		else if (!g_ascii_strcasecmp (str, "timestamp"))
+			type = GDA_TYPE_TIMESTAMP;
 		else if (!strcmp (str, "boolean"))
                         type = G_TYPE_BOOLEAN;
 		else if (!strcmp (str, "null"))
@@ -261,24 +267,29 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 
 /**
  * gda_utility_data_model_dump_data_to_xml
- * @model: 
+ * @model: a #GdaDataModel
+ * @parent: the parent XML node
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
  * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
- * @name: name to use for the XML resulting table.
+ * @use_col_ids:
  *
  * Dump the data in a #GdaDataModel into a xmlNodePtr (as used in libxml).
+ *
+ * Returns: TRUE if no error occurred
  */
-void
+gboolean
 gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
 					 const gint *cols, gint nb_cols, 
 					 const gint *rows, gint nb_rows,
 					 gboolean use_col_ids)
 {
+	gboolean retval = TRUE;
 	gint nrows, i;
 	gint *rcols, rnb_cols;
 	gchar **col_ids = NULL;
+	xmlNodePtr data = NULL;
 
 	/* compute columns if not provided */
 	if (!cols) {
@@ -317,43 +328,44 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 	else
 		nrows = nb_rows;
 	if (nrows > 0) {
-		xmlNodePtr row, data, field;
+		xmlNodePtr row;
 		gint r, c;
 
 		data = xmlNewChild (parent, NULL, (xmlChar*)"gda_array_data", NULL);
-		for (r = 0; r < nrows; r++) {
+		for (r = 0; (r < nrows) && retval; r++) {
 			row = xmlNewChild (data, NULL,  (xmlChar*)"gda_array_row", NULL);
 			for (c = 0; c < rnb_cols; c++) {
 				GValue *value;
 				gchar *str = NULL;
-				gboolean isnull = FALSE;
+				xmlNodePtr field = NULL;
 
-				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], rows ? rows [r] : r);
-				if (!value || gda_value_is_null ((GValue *) value)) 
-					isnull = TRUE;
-				else {
+				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], rows ? rows [r] : r, NULL);
+				if (!value) {
+					retval = FALSE;
+					break;
+				}
+				if (value && !gda_value_is_null ((GValue *) value)) { 
 					if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
 						str = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
 					else if (G_VALUE_TYPE (value) == G_TYPE_STRING) {
-						if (!g_value_get_string (value))
-							isnull = TRUE;
-						else
+						if (g_value_get_string (value))
 							str = gda_value_stringify (value);	
 					}
 					else
 						str = gda_value_stringify (value);
 				}
 				if (!use_col_ids) {
-					if (! str || (str && (*str == 0)))
-						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", NULL);
-					else
+					if (str && *str) 
 						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", (xmlChar*)str);
+					else
+						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", NULL);
 				}
 				else {
 					field = xmlNewChild (row, NULL,  (xmlChar*)"gda_array_value", (xmlChar*)str);
 					xmlSetProp(field, (xmlChar*)"colid",  (xmlChar*)col_ids [c]);
 				}
-				if (isnull)
+
+				if (!str)
 					xmlSetProp(field,  (xmlChar*)"isnull", (xmlChar*)"t");
 
 				g_free (str);
@@ -367,6 +379,13 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 			g_free (col_ids [c]);
 		g_free (col_ids);
 	}
+
+	if (!retval) {
+		xmlUnlinkNode (data);
+		xmlFreeNode (data);
+	}
+
+	return retval;
 }
 
 /**
@@ -374,14 +393,18 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
  * @holder:
  * @node: an xmlNodePtr with a &lt;parameter&gt; tag
  * @sources: a list of #GdaDataModel
+ * @error: a place to store errors, or %NULL
  *
- * WARNING: may set the "source" custom string property 
+ * Note: this method may set the "source" custom string property
+ *
+ * Returns: TRUE if no error occurred
  */
-void
-gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources)
+gboolean
+gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources, GError **error)
 {
 	xmlChar *str;
 	xmlNodePtr vnode;
+	gboolean retval = TRUE;
 
 	/* set properties from the XML spec */
 	str = xmlGetProp (node, BAD_CAST "id");
@@ -413,7 +436,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 	
 	str = xmlGetProp (node, BAD_CAST "source");
 	if (str) 
-		g_object_set_data_full (G_OBJECT (holder), "source", (gchar*)str, g_free);
+		g_object_set_data_full (G_OBJECT (holder), "source", str, xmlFree);
 
 	/* set restricting source if specified */
 	if (str && sources) {
@@ -443,7 +466,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 				    (fno >= gda_data_model_get_n_columns (model))) 
 					g_warning (_("Field number %d not found in source named '%s'"), fno, ptr1); 
 				else {
-					if (gda_holder_set_source_model (holder, model, fno, NULL)) {
+					if (gda_holder_set_source_model (holder, model, fno, error)) {
 						gchar *str;
 						/* rename the wrapper with the current holder's name */
 						g_object_get (G_OBJECT (holder), "name", &str, NULL);
@@ -451,6 +474,8 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 						g_object_get (G_OBJECT (holder), "description", &str, NULL);
 						g_object_set_data_full (G_OBJECT (model), "newdescr", str, g_free);
 					}
+					else
+						retval = FALSE;
 				}
 			}
 		}
@@ -460,15 +485,10 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 	vnode = node->children;
 	if (vnode) {
 		xmlChar *this_lang, *isnull;
-		const gchar *lang;
+		const gchar *lang = gda_lang_locale;
 		GType gdatype;
 
 		gdatype = gda_holder_get_g_type (holder);
-#ifdef HAVE_LC_MESSAGES
-		lang = setlocale (LC_MESSAGES, NULL);
-#else
-		lang = setlocale (LC_CTYPE, NULL);
-#endif
 		while (vnode) {
 			if (xmlNodeIsText (vnode)) {
 				vnode = vnode->next;
@@ -481,9 +501,9 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 			}
 
 			/* don't care about entries for the wrong locale */
-			this_lang = xmlGetProp(vnode, (xmlChar*)"lang");
+			this_lang = xmlGetProp (vnode, (xmlChar*)"lang");
 			if (this_lang && strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
-				g_free (this_lang);
+				xmlFree (this_lang);
 				vnode = vnode->next;
 				continue;
 			}
@@ -497,27 +517,23 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 			}
 			
 			if (!isnull) {
-				GValue *value;
 				gchar* nodeval = (gchar*)xmlNodeGetContent (vnode);
 
-				value = g_new0 (GValue, 1);
-				if (! gda_value_set_from_string (value, nodeval, gdatype)) {
-					/* error */
-					g_free (value);
-				}
-				else 
-					gda_holder_take_value (holder, value);
-
+				if (! gda_holder_set_value_str (holder, NULL, nodeval, error))
+					retval = FALSE;
  				xmlFree(nodeval);
 			}
 			else {
-				gda_holder_set_value (holder, NULL);
 				xmlFree (isnull);
+				if (! gda_holder_set_value (holder, NULL, error))
+					retval = FALSE;
 			}
 			
 			vnode = vnode->next;
 		}
 	}
+
+	return retval;
 }
 
 
@@ -650,8 +666,8 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 /*
  * Builds the WHERE condition of the computed DML statement
  */
-static GdaSqlExpr*
-dml_statements_build_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, gboolean require_pk, GError **error)
+GdaSqlExpr*
+gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, gboolean require_pk, GError **error)
 {
 	gint i;
 	GdaSqlExpr *expr;
@@ -672,16 +688,19 @@ dml_statements_build_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtab
 		}
 		else if (mtable->pk_cols_nb > 1) {
 			and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
-			and_cond->operator = GDA_SQL_OPERATOR_TYPE_AND;
+			and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
 			expr->cond = and_cond;
 		}
 		for (i = 0; i < mtable->pk_cols_nb; i++) {
 			GdaSqlSelectField *sfield = NULL;
 			GdaMetaTableColumn *tcol;
 			GSList *list;
+			gint index;
 			
 			tcol = (GdaMetaTableColumn *) g_slist_nth_data (mtable->columns, mtable->pk_cols_array[i]);
-			for (list = stsel->expr_list; list; list = list->next) {
+			for (index = 0, list = stsel->expr_list; 
+			     list; 
+			     index++, list = list->next) {
 				sfield = (GdaSqlSelectField *) list->data;
 				if (sfield->validity_meta_table_column == tcol)
 					break;
@@ -700,7 +719,7 @@ dml_statements_build_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtab
 
 				/* equal condition */
 				op = gda_sql_operation_new (GDA_SQL_ANY_PART (and_cond ? (gpointer)and_cond : (gpointer)expr));
-				op->operator = GDA_SQL_OPERATOR_TYPE_EQ;
+				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
 				if (and_cond) 
 					and_cond->operands = g_slist_append (and_cond->operands, op);
 				else
@@ -713,15 +732,19 @@ dml_statements_build_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtab
 				/* right operand */
 				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
 				pspec = g_new0 (GdaSqlParamSpec, 1);
-				pspec->name = g_strdup_printf ("-%d", mtable->pk_cols_array[i]);
+				pspec->name = g_strdup_printf ("-%d", index);
 				pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
-				pspec->type = g_strdup (gda_g_type_to_string (pspec->g_type));
 				pspec->nullok = tcol->nullok;
 				opexpr->param_spec = pspec;
 				op->operands = g_slist_append (op->operands, opexpr);
 			}
 		}
 	}
+	else {
+		TO_IMPLEMENT;
+		gda_sql_expr_free (expr);
+		expr = NULL;
+	}
 	return expr;
 	
  onerror:
@@ -788,9 +811,9 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 		GDA_SQL_ANY_PART (ust)->type = GDA_SQL_ANY_STMT_UPDATE;
 		ust->table = gda_sql_table_new (GDA_SQL_ANY_PART (ust));
 		ust->table->table_name = g_strdup ((gchar *) target->table_name);
-		ust->cond = dml_statements_build_condition (stsel, 
-							    GDA_META_TABLE (target->validity_meta_object),
-							    require_pk, error);
+		ust->cond = gda_compute_unique_table_row_condition (stsel, 
+								    GDA_META_TABLE (target->validity_meta_object),
+								    require_pk, error);
 		if (!ust->cond) {
 			retval = FALSE;
 			goto cleanup;
@@ -803,9 +826,9 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 		GDA_SQL_ANY_PART (dst)->type = GDA_SQL_ANY_STMT_DELETE;
 		dst->table = gda_sql_table_new (GDA_SQL_ANY_PART (dst));
 		dst->table->table_name = g_strdup ((gchar *) target->table_name);
-		dst->cond = dml_statements_build_condition (stsel, 
-							    GDA_META_TABLE (target->validity_meta_object),
-							    require_pk, error);
+		dst->cond = gda_compute_unique_table_row_condition (stsel, 
+								    GDA_META_TABLE (target->validity_meta_object),
+								    require_pk, error);
 		if (!dst->cond) {
 			retval = FALSE;
 			goto cleanup;
@@ -852,7 +875,6 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			GdaSqlParamSpec *pspec = g_new0 (GdaSqlParamSpec, 1);
 			pspec->name = g_strdup_printf ("+%d", colindex);
 			pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
-			pspec->type = g_strdup (gda_g_type_to_string (pspec->g_type));
 			pspec->nullok = tcol->nullok;
 			expr = gda_sql_expr_new (GDA_SQL_ANY_PART (ist));
 			expr->param_spec = pspec;
@@ -862,7 +884,6 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			GdaSqlParamSpec *pspec = g_new0 (GdaSqlParamSpec, 1);
 			pspec->name = g_strdup_printf ("+%d", colindex);
 			pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
-			pspec->type = g_strdup (gda_g_type_to_string (pspec->g_type));
 			pspec->nullok = tcol->nullok;
 			expr = gda_sql_expr_new (GDA_SQL_ANY_PART (ust));
 			expr->param_spec = pspec;
@@ -1073,6 +1094,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 {
 	GArray *compl = NULL;
 	gchar *text;
+	const GValue *cvalue;
 
 	if (!cnc) 
 		return NULL;
@@ -1150,15 +1172,18 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 		
 		nrows = gda_data_model_get_n_rows (model);
 		for (i = 0; i < nrows; i++) {
-			const gchar *tname;
-			tname = g_value_get_string (gda_data_model_get_value_at (model, 0, i));
-			if (!strncmp (tname, obj_name, len)) {
-				gchar *str;
-				if (schema_value) 
-					str = concat_ident (obj_schema, tname);
-				else
-					str = copy_ident (tname);
-				g_array_append_val (compl, str);
+			cvalue = gda_data_model_get_value_at (model, 0, i, NULL);
+			if (cvalue) {
+				const gchar *tname;
+				tname = g_value_get_string (cvalue);
+				if (!strncmp (tname, obj_name, len)) {
+					gchar *str;
+					if (schema_value) 
+						str = concat_ident (obj_schema, tname);
+					else
+						str = copy_ident (tname);
+					g_array_append_val (compl, str);
+				}
 			}
 		}
 		g_object_unref (model);
@@ -1177,12 +1202,15 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 		
 		nrows = gda_data_model_get_n_rows (model);
 		for (i = 0; i < nrows; i++) {
-			const gchar *cname;
-			cname = g_value_get_string (gda_data_model_get_value_at (model, 0, i));
-			if (!strncmp (cname, obj_name, len)) {
-				gchar *str;
-				str = copy_ident (cname);
-				g_array_append_val (compl, str);
+			cvalue = gda_data_model_get_value_at (model, 0, i, NULL);
+			if (cvalue) {
+				const gchar *cname;
+				cname = g_value_get_string (cvalue);
+				if (!strncmp (cname, obj_name, len)) {
+					gchar *str;
+					str = copy_ident (cname);
+					g_array_append_val (compl, str);
+				}
 			}
 		}
 		g_object_unref (model);
@@ -1200,30 +1228,35 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 		
 		nrows = gda_data_model_get_n_rows (model);
 		for (i = 0; i < nrows; i++) {
-			const gchar *tname;
-			tname = g_value_get_string (gda_data_model_get_value_at (model, 0, i));
-			if (!strncmp (tname, obj_name, len)) {
-				char *str;
-				GdaDataModel *m2;
-				str = copy_ident (tname);
+			cvalue = gda_data_model_get_value_at (model, 0, i, NULL);
+			if (cvalue) {
+				const gchar *tname;
+				tname = g_value_get_string (cvalue);
+				if (!strncmp (tname, obj_name, len)) {
+					char *str;
+					GdaDataModel *m2;
+					str = copy_ident (tname);
 				
-				m2 = gda_meta_store_extract (store, 
-							     "SELECT table_name FROM _tables WHERE table_schema = ##schema::string", 
-							     NULL, "schema", gda_data_model_get_value_at (model, 0, i), 
-							     NULL);
-				if (m2) {
-					gint i2, nrows2;
-					nrows2 = gda_data_model_get_n_rows (m2);
-					for (i2 = 0; i2 < nrows2; i2++) {
-						gchar *str2;
-						tname = g_value_get_string (gda_data_model_get_value_at (m2, 0, i2));
-						str2 = concat_ident (str, tname);
-						g_array_append_val (compl, str2);
+					m2 = gda_meta_store_extract (store, 
+								     "SELECT table_name FROM _tables WHERE table_schema = ##schema::string", 
+								     NULL, "schema", cvalue, NULL);
+					if (m2) {
+						gint i2, nrows2;
+						nrows2 = gda_data_model_get_n_rows (m2);
+						for (i2 = 0; i2 < nrows2; i2++) {
+							cvalue = gda_data_model_get_value_at (m2, 0, i2, NULL);
+							if (cvalue) {
+								gchar *str2;
+								tname = g_value_get_string (cvalue);
+								str2 = concat_ident (str, tname);
+								g_array_append_val (compl, str2);
+							}
+						}
+						
+						g_object_unref (m2);
 					}
-					
-					g_object_unref (m2);
+					free (str);
 				}
-				free (str);
 			}
 		}
 		g_object_unref (model);
@@ -1616,3 +1649,150 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
 	gda_rfc1738_decode (*out_username);
 	gda_rfc1738_decode (*out_password);
 }
+
+/**
+ * gda_parse_iso8601_date
+ * @gdate: a pointer to a #GDate structure which will be filled
+ * @value: a string
+ *
+ * Extracts date parts from @value, and sets @gdate's contents
+ *
+ * Accepted date format is "YYYY-MM-DD".
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_parse_iso8601_date (GDate *gdate, const gchar *value)
+{
+	GDateYear year;
+	GDateMonth month;
+	GDateDay day;
+
+	year = atoi (value);
+	value += 5;
+	month = atoi (value);
+	value += 3;
+	day = atoi (value);
+	
+	g_date_clear (gdate, 1);
+	if (g_date_valid_dmy (day, month, year)) {
+		g_date_set_dmy (gdate, day, month, year);
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
+
+/**
+ * gda_parse_iso8601_time
+ * @timegda: a pointer to a #GdaTime structure which will be filled
+ * @value: a string
+ *
+ * Extracts time parts from @value, and sets @timegda's contents
+ *
+ * Accepted date format is "HH:MM:SS[.ms][TZ]" where TZ is +hour or -hour
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_parse_iso8601_time (GdaTime *timegda, const gchar *value)
+{
+	timegda->hour = atoi (value);
+	value += 3;
+	timegda->minute = atoi (value);
+	value += 3;
+	timegda->second = atoi (value);
+	value += 2;
+	if (*value != '.') {
+		timegda->fraction = 0;
+	} else {
+		gint ndigits = 0;
+		gint64 fraction;
+
+		value++;
+		fraction = atol (value);
+		while (*value && *value != '+') {
+			value++;
+			ndigits++;
+		}
+
+		while (ndigits < 3) {
+			fraction *= 10;
+			ndigits++;
+		}
+
+		while (fraction > 0 && ndigits > 3) {
+			fraction /= 10;
+			ndigits--;
+		}
+		
+		timegda->fraction = fraction;
+	}
+
+	if (*value)
+		timegda->timezone = atol (value) * 60 * 60;
+	else
+		timegda->timezone = 0;
+
+	return TRUE;
+}
+
+/**
+ * gda_parse_iso8601_timestamp
+ * @timestamp: a pointer to a #GdaTimeStamp structure which will be filled
+ * @value: a string
+ *
+ * Extracts date and time parts from @value, and sets @timestamp's contents
+ *
+ * Accepted date format is "YYYY-MM-DD HH:MM:SS[.ms][TZ]" where TZ is +hour or -hour
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_parse_iso8601_timestamp (GdaTimestamp *timestamp, const gchar *value)
+{
+	timestamp->year = atoi (value);
+	value += 5;
+	timestamp->month = atoi (value);
+	value += 3;
+	timestamp->day = atoi (value);
+	value += 3;
+	timestamp->hour = atoi (value);
+	value += 3;
+	timestamp->minute = atoi (value);
+	value += 3;
+	timestamp->second = atoi (value);
+	value += 2;
+	if (*value != '.') {
+		timestamp->fraction = 0;
+	} else {
+		gint ndigits = 0;
+		gint64 fraction;
+
+		value++;
+		fraction = atol (value);
+		while (*value && *value != '+') {
+			value++;
+			ndigits++;
+		}
+
+		while (ndigits < 3) {
+			fraction *= 10;
+			ndigits++;
+		}
+
+		while (fraction > 0 && ndigits > 3) {
+			fraction /= 10;
+			ndigits--;
+		}
+		
+		timestamp->fraction = fraction;
+	}
+
+	if (*value)
+		timestamp->timezone = atol (value) * 60 * 60;
+	else
+		timestamp->timezone = 0;
+
+	return TRUE;
+}
